;----------------------------------------------------------------------------

;Title  : DDS-based strobe
;Version: 1.20
;Target : ATtiny12
;Author : wubblick@yahoo.com

;AVR Assembler 2.0

;Fuse bits: 0x1A

;----------------------------------------------------------------------------

.include "tn12def.inc"

;----------------------------------------------------------------------------

;Constants:

#define F_CLK 4000000   ;Fclk, Hz
#define DIV_N 256       ;clk to overflow Timer0

#define FREQ(RPM,LINES) (DIV_N * 0xFFFFFF * RPM * LINES / (60000 * F_CLK))

;----------------------------------------------------------------------------

;Frequency Presets:

;Nominal speed  50Hz region  Strobe (50Hz) 60Hz region  Strobe (60Hz)
;
;33 1/3 RPM     33.333 RPM   180 lines     33.333 RPM   216 lines
;45 RPM         45.113 RPM   133 lines     45.000 RPM   160 lines
;78 RPM         77.922 RPM    77 lines     78.261 RPM    92 lines

#define FREQ_1 FREQ(33333, 180) ;preset 1
#define FREQ_2 FREQ(45000, 133) ;preset 2
#define FREQ_3 FREQ(78000, 180) ;preset 3
#define FREQ_4 FREQ(78000, 133) ;preset 4

;#define FREQ_4 FREQ(33333, 216) ;preset 1
;#define FREQ_3 FREQ(45000, 160) ;preset 2
;#define FREQ_1 FREQ(45000, 216) ;preset 3
;#define FREQ_2 FREQ(78261, 216) ;preset 4

;----------------------------------------------------------------------------

;Ports definition:

.equ LED    = PB0 ;LED output, active high
.equ SEL0   = PB1 ;frequency select 0
.equ SEL1   = PB2 ;frequency select 1

;Port B direction:
.equ DIRB   = (1 << LED)
;Port B init/pullups:
.equ PUPB   = ~(1 << LED)

;----------------------------------------------------------------------------

;Global register variables:

.def    tsreg   = r2  ;SREG store

.def    Phase1  = r3  ;phase code
.def    Phase2  = r4
.def    Phase3  = r5

.def    Freq1   = r16 ;frequency code (delta phase)
.def    Freq2   = r17
.def    Freq3   = r18

.def    Tempi   = r19

.def    Temp    = r20
.def    Select  = r21 ;frequency set

;----------------------------------------------------------------------------

.CSEG
.org 0

        rjmp    Init            ;reset vector

;----------------------------------------------------------------------------

.org OVF0addr

;Timer0 overflow interrupt vector (DDS implementation)
;To minimize execution time handle is located
;at vector address. It eliminates RJMP command.

                                ;4   address store
        in      tsreg,SREG      ;1   save status register

        add     Phase1,Freq1    ;1   Phase = Phase + Freq
        adc     Phase2,Freq2    ;1
        adc     Phase3,Freq3    ;1

        mov     Tempi,Phase3    ;1
        andi    Tempi,0xF0      ;1
        brne    Off             ;1/2
On:     sbi     PORTB,LED       ;2   LED on
        rjmp    OnOff           ;2
Off:    cbi     PORTB,LED       ;2   LED off
        nop                     ;1

OnOff:  out     SREG,tsreg      ;1   restore status register
        reti                    ;4 (20 total)

;----------------------------------------------------------------------------

;Timer0 init:

Init:   ldi     Temp,(1<<CS00)  
        out     TCCR0,Temp      ;CK/1
        ldi     Temp,(1<<TOIE0) 
        out     TIMSK,Temp;timer OVF interrupt enable

;Port init:

        ldi     Temp,PUPB
        out     PORTB,Temp
        ldi     Temp,DIRB       
        out     DDRB,Temp

;Clear frequency set:

        ldi Select,0xFF

;Interrupts enable:

        sei

;Main program:

Main:   clr     Temp
        sbis    PINB,SEL0
        ori     Temp,0x01
        sbis    PINB,SEL1
        ori     Temp,0x02
        cp      Temp,Select
        breq    Main

        mov     Select,Temp

        cpi     Select,0
        brne    Sel_1
        ldi Freq1,byte1(FREQ_1)
        ldi Freq2,byte2(FREQ_1)
        ldi Freq3,byte3(FREQ_1)
        rjmp Sel_4

Sel_1:  cpi     Select,1
        brne    Sel_2
        ldi Freq1,byte1(FREQ_2)
        ldi Freq2,byte2(FREQ_2)
        ldi Freq3,byte3(FREQ_2)
        rjmp Sel_4

Sel_2:  cpi     Select,2
        brne    Sel_3
        ldi Freq1,byte1(FREQ_3)
        ldi Freq2,byte2(FREQ_3)
        ldi Freq3,byte3(FREQ_3)
        rjmp Sel_4

Sel_3:  cpi     Select,3
        brne    Sel_4
        ldi Freq1,byte1(FREQ_4)
        ldi Freq2,byte2(FREQ_4)
        ldi Freq3,byte3(FREQ_4)

Sel_4:  rjmp Main

;----------------------------------------------------------------------------
